diff --git a/web-bundle/src/main/java/com/graphhopper/resources/MapMatchingResource.java b/web-bundle/src/main/java/com/graphhopper/resources/MapMatchingResource.java
index 65422f8e4..55ca53492 100644
--- a/web-bundle/src/main/java/com/graphhopper/resources/MapMatchingResource.java
+++ b/web-bundle/src/main/java/com/graphhopper/resources/MapMatchingResource.java
@@ -23,6 +23,7 @@ import com.fasterxml.jackson.databind.node.JsonNodeFactory;
 import com.fasterxml.jackson.databind.node.ObjectNode;
 import com.graphhopper.GHResponse;
 import com.graphhopper.GraphHopper;
+import com.graphhopper.OSMGraphHopper;
 import com.graphhopper.ResponsePath;
 import com.graphhopper.jackson.ResponsePathSerializer;
 import com.graphhopper.matching.*;
@@ -52,12 +53,12 @@ public class MapMatchingResource {
 
     private static final Logger logger = LoggerFactory.getLogger(MapMatchingResource.class);
 
-    private final GraphHopper graphHopper;
+    private final OSMGraphHopper graphHopper;
     private final ProfileResolver profileResolver;
     private final TranslationMap trMap;
 
     @Inject
-    public MapMatchingResource(GraphHopper graphHopper, ProfileResolver profileResolver, TranslationMap trMap) {
+    public MapMatchingResource(OSMGraphHopper graphHopper, ProfileResolver profileResolver, TranslationMap trMap) {
         this.graphHopper = graphHopper;
         this.profileResolver = profileResolver;
         this.trMap = trMap;
@@ -153,7 +154,8 @@ public class MapMatchingResource {
                         build();
             } else {
                 ObjectNode map = ResponsePathSerializer.jsonObject(rsp, instructions, calcPoints, enableElevation, pointsEncoded, took);
-
+                ArrayNode osmWayIds = map.putArray("osmWayIds");
+                addOsmWayIds(matchResult, osmWayIds);
                 Map<String, Object> matchStatistics = new HashMap<>();
                 matchStatistics.put("distance", matchResult.getMatchLength());
                 matchStatistics.put("time", matchResult.getMatchMillis());
@@ -176,6 +178,23 @@ public class MapMatchingResource {
         }
     }
 
+    private void addOsmWayIds(MatchResult matchResult, ArrayNode osmWayIds) {
+        List<Long> addedWayIds = new ArrayList<>();
+        for(EdgeMatch match : matchResult.getEdgeMatches()) {
+            try {
+                int internalEdgeId = match.getEdgeState().getEdge();
+                Long osmWayId = graphHopper.getOSMWay(internalEdgeId);
+                if(!addedWayIds.contains(osmWayId)) {
+                    addedWayIds.add(osmWayId);
+                    osmWayIds.add(osmWayId);
+                }
+            } catch (Exception e) {
+                System.out.println(e);
+                e.printStackTrace();
+            }
+        }
+    }
+
     private void removeLegacyParameters(PMap hints) {
         hints.remove("vehicle");
         hints.remove("weighting");
